using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace Chilkat
{
    public class CkObject : IDisposable
    {
        // ----- CkByteData functions. ------
        [DllImport(ChilkatConst.DllPath, EntryPoint = "CkByteData_CreateA", CharSet = CharSet.Ansi)]
        private static extern System.IntPtr CkByteData_CreateA();

        [DllImport(ChilkatConst.DllPath, EntryPoint = "CkByteData_DisposeA", CharSet = CharSet.Ansi)]
        private static extern void CkByteData_DisposeA(System.IntPtr pObj);

        [DllImport(ChilkatConst.DllPath, EntryPoint = "CkByteData_getSize", CharSet = CharSet.Ansi)]
        private static extern uint CkByteData_getSize(System.IntPtr pObj);

        [DllImport(ChilkatConst.DllPath, EntryPoint = "CkByteData_getData", CharSet = CharSet.Ansi)]
        private static extern System.IntPtr CkByteData_getData(System.IntPtr pObj);

        [DllImport(ChilkatConst.DllPath, EntryPoint = "CkByteData_borrowData", CharSet = CharSet.Ansi)]
        private static extern void CkByteData_borrowData(System.IntPtr pObj, System.IntPtr pBytes, uint numBytes);

        // ----- END OF CkByteData functions. ------


        public CkObject()
        {
        }

        virtual protected void disposeUnmanaged()
        {
        }
        
        public void Dispose()
        {
            disposeUnmanaged();
            GC.SuppressFinalize(this);
        }

        public System.IntPtr newCkByteData()
        {
            return CkByteData_CreateA();
        }
        public void disposeCkByteData(System.IntPtr hByteData)
        {
            CkByteData_DisposeA(hByteData);
        }

        // Return the unmanaged bytes contained within hByteData as a managed byte array.
        public byte[] ckToBytes(System.IntPtr hByteData)
        {
            if (hByteData == System.IntPtr.Zero) return new byte[0];

            byte[] retBytes = null;
            uint numResultBytes = CkByteData_getSize(hByteData);
            if (numResultBytes > 0)
            {
                System.IntPtr pUnmanagedBytes = CkByteData_getData(hByteData);
                try
                {
                    retBytes = new byte[numResultBytes];
                    Marshal.Copy(pUnmanagedBytes, retBytes, 0, (int)numResultBytes);
                }
                catch { }
            }
            else
            {
                retBytes = new byte[0];
            }
            return retBytes;
        }

        // Marshal to managed bytes to unmanaged bytes, and let hByteData borrow
        // the unmanaged bytes.  Return the unmanaged bytes so they can be deallocated later by the caller.
        public System.IntPtr ckFromBytes(byte[] data, System.IntPtr hByteData)
        {
            if (data == null) return System.IntPtr.Zero;

            System.IntPtr pUnmanagedBytes = System.IntPtr.Zero;

            int szData = data.Length;
            if (szData == 0) return pUnmanagedBytes;

            try
            {
                pUnmanagedBytes = Marshal.AllocHGlobal(szData);
                Marshal.Copy(data, 0, pUnmanagedBytes, szData);

                if (hByteData != System.IntPtr.Zero)
                {
                    CkByteData_borrowData(hByteData, pUnmanagedBytes, (uint)szData);
                }
            }
            catch { }

            return pUnmanagedBytes;
        }

        // From an unmanaged utf-16 string to a managed string.
        public string ckFromUni(System.IntPtr pUni)
        {
            if (pUni == System.IntPtr.Zero) return null;
            return Marshal.PtrToStringUni(pUni);
        }



    }

}

