using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace Chilkat
{
	public class ServerSideEvent : CkObject
	{
		private bool m_needDisposal = false;
		private System.IntPtr m_pObj;

		[DllImport(ChilkatConst.DllPath, EntryPoint = "CkServerSideEventU_CreateW", CharSet = CharSet.Unicode)]
		private static extern System.IntPtr CkServerSideEventU_CreateW();

		[DllImport(ChilkatConst.DllPath, EntryPoint = "CkServerSideEventU_DisposeW", CharSet = CharSet.Unicode)]
		private static extern void CkServerSideEventU_DisposeW(System.IntPtr pObj);

        [DllImport(ChilkatConst.DllPath, EntryPoint = "CkServerSideEventU_dataW", CharSet = CharSet.Unicode)]
        private static extern System.IntPtr CkServerSideEventU_dataW(System.IntPtr pObj);
        [DllImport(ChilkatConst.DllPath, EntryPoint = "CkServerSideEventU_eventNameW", CharSet = CharSet.Unicode)]
        private static extern System.IntPtr CkServerSideEventU_eventNameW(System.IntPtr pObj);
        [DllImport(ChilkatConst.DllPath, EntryPoint = "CkServerSideEventU_lastEventIdW", CharSet = CharSet.Unicode)]
        private static extern System.IntPtr CkServerSideEventU_lastEventIdW(System.IntPtr pObj);
        [DllImport(ChilkatConst.DllPath, EntryPoint = "CkServerSideEventU_getRetryW", CharSet = CharSet.Unicode)]
        private static extern int CkServerSideEventU_getRetryW(System.IntPtr pObj);
        [DllImport(ChilkatConst.DllPath, EntryPoint = "CkServerSideEventU_LoadEventW", CharSet = CharSet.Unicode)]
        private static extern int CkServerSideEventU_LoadEventW(System.IntPtr pObj, string eventText);


		public ServerSideEvent()
		{
			m_pObj = CkServerSideEventU_CreateW();
			m_needDisposal = true;
		}

		public ServerSideEvent(System.IntPtr p)
		{
			m_pObj = p;
			m_needDisposal = true;
		}

		override protected void disposeUnmanaged()
		{
			if (m_needDisposal)
			{
				m_needDisposal = false;
				System.IntPtr p = m_pObj;
				m_pObj = System.IntPtr.Zero;
				CkServerSideEventU_DisposeW(p);
			}
		}

		~ServerSideEvent()
		{
			disposeUnmanaged();
		}

		public System.IntPtr getImpl()
		{
			return  m_pObj;
		}

		// This class does not have events.
		// dnClassName = ServerSideEvent
	/// <summary>
	/// The data for the server-side event. (If the "data" field was empty, then this
	/// will be empty.)
	/// </summary>
	public string Data
	{
		get
		{
			return ckFromUni(CkServerSideEventU_dataW(m_pObj));
		}
	}

	/// <summary>
	/// The name of the server-side event. (If the "event" field was not present, then
	/// this will be empty.)
	/// </summary>
	public string EventName
	{
		get
		{
			return ckFromUni(CkServerSideEventU_eventNameW(m_pObj));
		}
	}

	/// <summary>
	/// The content of the "id" field, if present.
	/// </summary>
	public string LastEventId
	{
		get
		{
			return ckFromUni(CkServerSideEventU_lastEventIdW(m_pObj));
		}
	}

	/// <summary>
	/// The integer value of the "retry" field, if present. Otherwise 0.
	/// </summary>
	public int Retry
	{
		get
		{
			return CkServerSideEventU_getRetryW(m_pObj);
		}
	}

	/// <summary>
	/// Loads the multi-line event text into this object. For example, the eventText for a
	/// Firebase event might look like this:
	/// event: put
	/// data: {"path": "/c", "data": {"foo": true, "bar": false}}
	///
	/// </summary>
	public bool LoadEvent(string eventText)
	{
		int result = CkServerSideEventU_LoadEventW(m_pObj, eventText);
		return result == 0 ? false : true;
	}


	}
}

