using System;

public class ChilkatTest
{
	private Chilkat.Task m_task = null;
	
    private void logProgressInfo(string name, string value)
        {
            Console.WriteLine("(ProgressInfo) " + name + ": " + value + "\r\n");
        }
        
    // The AbortCheck event is called periodically according to the HeartbeatMs property.
    // A PercentDone event takes the place of an AbortCheck because it has the ability
    // to abort.  Therefore, in this example, AbortCheck may never actually be called.
    private void doAbortCheck(out bool abort)
    {
    	Console.WriteLine("AbortCheck.\r\n");
    	
        // To abort the Chilkat method call, set abort = true.
        abort = false;
    }

	private void handleTaskIdCompleted(int taskId)
	{
		if (m_task == null)
		{
			Console.WriteLine("m_task is null!\r\n");
			return;
		}
		if (m_task.TaskId != taskId)
		{
			Console.WriteLine("Unexpected task ID!\r\n");
			return;
		}
		
		Console.WriteLine("Task completed!\r\n");
	}
	
	private void testHttpGet()
	{
		Chilkat.Http http = new Chilkat.Http();
		bool success = http.UnlockComponent("30-day trial");
		if (!success)
		{
			Console.WriteLine(http.LastErrorText);
			return;
		}
		
        Chilkat.Http.ProgressInfo progressInfo = new Chilkat.Http.ProgressInfo(logProgressInfo);
        http.setProgressInfoCb(progressInfo);

        http.HeartbeatMs = 200;  // call every .2 seconds
        Chilkat.Http.AbortCheck abortCheck = new Chilkat.Http.AbortCheck(doAbortCheck);
        http.setAbortCheckCb(abortCheck);

		string html = http.QuickGetStr("http://www.chilkatsoft.com/");
        if (html == null)
            {
            Console.WriteLine(http.LastErrorText);
            return;
            }

        Console.WriteLine("success.\n");

	}

	private void testHttpGetAsync()
	{
		Chilkat.Http http = new Chilkat.Http();
		bool success = http.UnlockComponent("30-day trial");
		if (!success)
		{
			Console.WriteLine(http.LastErrorText);
			return;
		}
		
		// Note: Events are called from the background thread for asynchronous method calls.
        Chilkat.Http.ProgressInfo progressInfo = new Chilkat.Http.ProgressInfo(logProgressInfo);
        http.setProgressInfoCb(progressInfo);

        http.HeartbeatMs = 200;  // call every .2 seconds
        Chilkat.Http.AbortCheck abortCheck = new Chilkat.Http.AbortCheck(doAbortCheck);
        http.setAbortCheckCb(abortCheck);

		Chilkat.Http.TaskIdCompleted taskIdCompleted = new Chilkat.Http.TaskIdCompleted(handleTaskIdCompleted);
		http.setTaskIdCompletedCb(taskIdCompleted);
		
		m_task = http.QuickGetStrAsync("http://www.chilkatsoft.com/");
        if (m_task == null)
            {
            Console.WriteLine(http.LastErrorText);
            return;
            }


		//  Schedule the task for running on the thread pool.  This changes the task's state
		//  from Inert to Live.
		success = m_task.Run();
		if (success != true) {
		    Console.WriteLine(m_task.LastErrorText);
		    return;
		}

		//  The application is now free to do anything else
		//  while the HTTP operation is in progress.

		//  For this example, we'll simply sleep and periodically
		//  check to see if the task if finished.  
		while (!m_task.Finished) {

		    //  Sleep 100 ms.
		    m_task.SleepMs(100);

		}

		//  A finished task could be one that was canceled, aborted, or truly finished.

		//  If the task was "canceled", it was canceled prior to actually starting.  This could
		//  happen if the task was canceled while waiting in a thread pool queue to be scheduled by Chilkat's
		//  background thread pool scheduler.

		//  If the task was "aborted", it indicates that it was canceled while running in a background thread.
		//  The ResultErrorText will likely indicate that the task was aborted.

		//  If the task "completed", then it ran to completion, but the actual success/failure of the method
		//  is determined by the result obtained via a GetResult* method.  (A "completed" task will
		//  have a StatusInt equal to 7.   If the task finished, but was not completed, then it must've
		//  been aborted or canceled:
		if (m_task.StatusInt != 7) {
		    Console.WriteLine("Task did not complete.");
		    Console.WriteLine("task status: " + m_task.Status);

		    return;
		}

		//  The QuickGetStr method returns a string.  Therefore, after the task is finished,
		//  we can get the string result by calling GetResultString.  This is the return value had
		//  we called QuickGetStr synchronously.
		string html = m_task.GetResultString();
		if (html == null) {
		    //  The task's ResultErrorText contains what would have been in the LastErrorText property had
		    //  the SendEmail method been called synchronously.
		    Console.WriteLine(m_task.ResultErrorText);
		    return;
		}

        Console.WriteLine("success.\n");

	}

	private void testXml()
	{
		Chilkat.Xml xml = new Chilkat.Xml();

    	Console.WriteLine("Chilkat version: " + xml.Version);
        
        xml.Tag = "Test";
        xml.Content = "content";
        xml.SaveXml("out.xml");
	}

    private void testCrypt()
    {
        Chilkat.Crypt2 crypt = new Chilkat.Crypt2();

        bool success = crypt.UnlockComponent("30-day trial.");
        if (!success)
        {
            Console.WriteLine(crypt.LastErrorText);
            return;
        }

        string plainText = "The quick brown fox jumped over the lazy dog";

        crypt.CryptAlgorithm = "aes";
        crypt.KeyLength = 256;
        crypt.CipherMode = "cbc";
        crypt.SetEncodedIV("000102030405060708090A0B0C0D0E0F", "hex");
        crypt.SetEncodedKey("000102030405060708090A0B0C0D0E0F000102030405060708090A0B0C0D0E0F", "hex");

        crypt.EncodingMode = "hex";
        string cipherText = crypt.EncryptStringENC(plainText);

        Console.WriteLine("cipherText in hex encoding:");
        Console.WriteLine(cipherText);

        string restoredText = crypt.DecryptStringENC(cipherText);
        Console.WriteLine("restored text: " + restoredText);

        // Now do the same in a more complicated way using byte arrays.
        byte[] quickFoxBytes = System.Text.Encoding.UTF8.GetBytes(plainText);
        byte[] cipherBytes = crypt.EncryptBytes(quickFoxBytes);

        Console.WriteLine("--");
        Console.WriteLine("cipherText in hex encoding:");
        Console.WriteLine(crypt.Encode(cipherBytes, "hex"));

        byte[] decryptedBytes = crypt.DecryptBytes(cipherBytes);
        restoredText = System.Text.Encoding.UTF8.GetString(decryptedBytes);
        Console.WriteLine("restored text: " + restoredText);
    }

	static public void Main ()
	{
		ChilkatTest test = new ChilkatTest();
		//test.testXml();
		//test.testHttpGet();
		//test.testHttpGetAsync();
        test.testCrypt();
	}
 
}

